import json
import logging
import config as cfg
import os
import threading
import time
from multiprocessing import Process
from queue import Queue
import uuid
from confluent_kafka import Consumer
import time
import random

# settings
broker = cfg.broker
topic = cfg.topic
groupId = cfg.groupId
ssl_cafile = cfg.ssl_cafile
ssl_certfile = cfg.ssl_certfile
ssl_keyfile = cfg.ssl_keyfile
ssl_password = cfg.ssl_password

conf = {
    'bootstrap.servers': ','.join(broker),
        'group.id': groupId,
        'session.timeout.ms': 6000,
        # 'on_commit': my_commit_callback,
        'auto.offset.reset': 'earliest',
        'security.protocol': 'SSL',
        'ssl.certificate.location': ssl_certfile,
        'ssl.ca.location': ssl_cafile,
        'ssl.key.location': ssl_keyfile,
        'ssl.key.password': ssl_password,
        'enable.ssl.certificate.verification': 'false',
        }

def _process_msg(q, c):
    msg = q.get(timeout=60)
    logging.info(
        '#%sT%s - Received message: %s',
        os.getpid(), threading.get_ident(), json.loads(msg.value().decode('utf-8'))['index']
    )
    # time.sleep(0.5)
    q.task_done()
    c.commit(msg)
def _consume(config):
    logging.info(
        '#%s - Starting consumer group=%s, topic=%s',
        os.getpid(), config['kafka_kwargs']['group.id'], config['topic'],
    )
    c = Consumer(**config['kafka_kwargs'])
    c.subscribe([config['topic']])
    q = Queue(maxsize=config['num_threads'])

    while True:
        logging.info('#%s - Waiting for message...', os.getpid())
        try:
            msg = c.poll(60)
            if msg is None:
                continue
            if msg.error():
                logging.error(
                    '#%s - Consumer error: %s', os.getpid(), msg.error()
                )
                continue
            q.put(msg)
            # Use default daemon=False to stop threads gracefully in order to
            # release resources properly.
            t = threading.Thread(target=_process_msg, args=(q, c))
            t.start()
        except Exception:
            logging.exception('#%s - Worker terminated.', os.getpid())
            c.close()

def main(config):
    """
    Simple program that consumes messages from Kafka topic and prints to
    STDOUT.
    """
    time.sleep(random.uniform(1, 60))

    workers = []
    while True:
        num_alive = len([w for w in workers if w.is_alive()])
        if config['num_workers'] == num_alive:
            continue
        for _ in range(config['num_workers']-num_alive):
            consumerGroup = str(uuid.uuid4())
            config['kafka_kwargs']['group.id'] = 'test-group-' + consumerGroup
            print(config['kafka_kwargs']['group.id'])
            p = Process(target=_consume, daemon=True, args=(config,))
            p.start()
            workers.append(p)
            logging.info('Starting worker #%s', p.pid)


if __name__ == '__main__':
    logging.basicConfig(
        level=getattr(logging, os.getenv('LOGLEVEL', '').upper(), 'INFO'),
        format='[%(asctime)s] %(levelname)s:%(name)s:%(message)s',
    )
    main(config={
        # At most, this should be the total number of Kafka partitions on
        # the topic.
        'num_workers': 16,
        'num_threads': 16,
        'topic': cfg.topic,
        'kafka_kwargs': conf,
    })
