import json

from kafka import KafkaConsumer, KafkaProducer, TopicPartition
from datetime import datetime
import uuid
import sys
import base64
import config as cfg

# settings
broker = cfg.broker
topic = cfg.topic
ssl_cafile = cfg.ssl_cafile
ssl_certfile = cfg.ssl_certfile
ssl_keyfile = cfg.ssl_keyfile
ssl_password = cfg.ssl_password
image = cfg.image
testing_results = "../testing-results-" + datetime.now().strftime("%d%m%Y-%H%M") + ".txt"

nbrrecords = int(sys.argv[1])
structured = sys.argv[2] == 1 or sys.argv[2] == '1'
nbrrecordsinserted = int(0)
nbrrecordsretreived = int(0)

payload = {}
encoded_image = ''

# Open the image file
with open(image, "rb") as f:
    encoded_image = base64.b64encode(f.read())

print("Testing the performance of the Kafka cluster for " + str(nbrrecords) + " records and structured = " + str(
    structured))


def generate_report(nbrrecords, nbrrecordsinserted, nbrrecordsretreived):
    with open(testing_results, 'w') as f:
        f.write("Number of record to insert: " + str(nbrrecords))
        f.write("\n")
        f.write("Number of record inserted: " + str(nbrrecordsinserted))
        f.write("\n")
        f.write("Number of record consumed: " + str(nbrrecordsretreived))


try:
    producer = KafkaProducer(security_protocol="SSL",
                             bootstrap_servers=broker,
                             ssl_cafile=ssl_cafile,
                             ssl_certfile=ssl_certfile,
                             ssl_keyfile=ssl_keyfile,
                             ssl_check_hostname=False,
                             ssl_password=ssl_password,
                             value_serializer=lambda v: json.dumps(v).encode('ascii'),
                             key_serializer=lambda k: str(k).encode('ascii')
                             )
    print("Generating the " + str(nbrrecords) + " records")
    for i in range(1, nbrrecords + 1):
        key = uuid.uuid4()
        payload = {"index": i, "value": str(uuid.uuid4()), "timestamp": datetime.now().strftime("%d%m%Y-%H:%M:%S.%f")}
        if not structured:
            payload = {"index": i, "value": str(encoded_image),
                       "timestamp": datetime.now().strftime("%d%m%Y-%H:%M:%S.%f")}
            # print(payload)
        producer.send(topic, key=key, value=payload)
        nbrrecordsinserted = i
except Exception as e:
    with open(testing_results, 'w') as f:
        f.write("ERROR : Broker not available while inserting record " + str(i) + " !")
        f.write("\n")
    generate_report(nbrrecords, nbrrecordsinserted, nbrrecordsretreived)
    sys.exit(1)

print("End of the generation")

try:
    # prepare consumer
    print("Preparing the consumer")
    tp = TopicPartition(topic, 0)
    consumer = KafkaConsumer(security_protocol="SSL",
                             bootstrap_servers=broker,
                             ssl_cafile=ssl_cafile,
                             ssl_certfile=ssl_certfile,
                             ssl_keyfile=ssl_keyfile,
                             ssl_check_hostname=False,
                             ssl_password=ssl_password,
                             request_timeout_ms=20000,
                             group_id='test-group',
                             auto_offset_reset='earliest',
                             enable_auto_commit=True,
                             auto_commit_interval_ms=1000,
                             )
    consumer.assign([tp])
    consumer.seek_to_beginning(tp)

    # obtain the last offset value
    lastOffset = consumer.end_offsets([tp])[tp]

    # consume the messages
    for message in consumer:
        # print "Offset:", message.offset
        # print "Value:", message.value

        nbrrecordsretreived += 1

        if message.offset == lastOffset - 1:
            break

    with open(testing_results, 'w') as f:
        f.write("Consume process completed")
        f.write("\n")
    generate_report(nbrrecords, nbrrecordsinserted, nbrrecordsretreived)
except Exception as error:
    print("An exception occurred: ", error)
    with open(testing_results, 'w') as f:
        f.write("ERROR during consume process !")
        f.write("\n")
    generate_report(nbrrecords, nbrrecordsinserted, nbrrecordsretreived)

if nbrrecordsinserted == nbrrecordsretreived:
    sys.exit(0)
else:
    sys.exit(1)
