import json

from kafka import KafkaConsumer, KafkaProducer, TopicPartition
from datetime import datetime
import uuid
import sys
import base64
import config as cfg

# settings
broker = cfg.broker
topic = cfg.topic
ssl_cafile = cfg.ssl_cafile
ssl_certfile = cfg.ssl_certfile
ssl_keyfile = cfg.ssl_keyfile
ssl_password = cfg.ssl_password
image = cfg.image

nbrrecords = int(sys.argv[1])
structured = sys.argv[2] == 1 or sys.argv[2] == '1'
nbrrecordsinserted = int(0)
nbrrecordsretreived = int(0)

payload = {}
encoded_image = ''

# Open the image file
with open(image, "rb") as f:
    encoded_image = base64.b64encode(f.read())

print("Testing the performance of the Kafka cluster for " + str(nbrrecords) + " records and structured = " + str(
    structured))


def on_send_success(record_metadata):
    print(record_metadata.topic)
    print(record_metadata.partition)
    print(record_metadata.offset)


def on_send_error(excp):
    print('I am an errback', exc_info=excp)
    # handle exception


try:
    producer = KafkaProducer(security_protocol="SSL",
                             bootstrap_servers=broker,
                             ssl_cafile=ssl_cafile,
                             ssl_certfile=ssl_certfile,
                             ssl_keyfile=ssl_keyfile,
                             ssl_check_hostname=False,
                             ssl_password=ssl_password,
                             value_serializer=lambda v: json.dumps(v).encode('ascii'),
                             key_serializer=lambda k: str(k).encode('ascii'),
                             )

    print("Sending the " + str(nbrrecords) + " records, " )
    for i in range(1, nbrrecords + 1):
        key = uuid.uuid4()
        payload = {"index": i, "value": str(uuid.uuid4()), "timestamp": datetime.now().strftime("%d%m%Y-%H:%M:%S.%f")}
        if not structured:
            payload = {"index": i, "value": str(encoded_image),
                       "timestamp": datetime.now().strftime("%d%m%Y-%H:%M:%S.%f")}

        res = producer.send(topic, key=key, value=payload)
        # res = producer.send(topic, key=key, value=payload).add_callback(on_send_success).add_errback(on_send_error)

        nbrrecordsinserted = i
    producer.flush()

    # block until all async messages are sent


    # print("Generated: " + nbrrecords)
    # print("Inserted: " + nbrrecordsinserted)
except Exception as e:
    print("An exception occurred: ", e)
    sys.exit(1)

print("End of the sending of the " + str(nbrrecords) + " records")
